@@ -8,7 +8,7 @@ use core::ops::Index;
 
 use super::arena::NodeArena;
 use super::error::SGErr;
-use super::iter::{ConsumingIter, Iter, IterMut};
+use super::iter::{IntoIter, Iter, IterMut};
 use super::node::{Node, NodeGetHelper, NodeRebuildHelper};
 use super::types::{
     Idx, IdxVec, RebuildMetaVec, SortMetaVec, SortNodeRefIdxPairVec, SortNodeRefVec,
@@ -27,7 +27,7 @@ use smallvec::smallvec;
 pub struct SGTree<K: Ord, V> {
     // Storage
     pub(crate) arena: NodeArena<K, V>,
-    pub(crate) root_idx: Option<Idx>,
+    pub(crate) root_idx: Option<Idx>, // TODO: rename to opt_root_idx
 
     // Query cache
     max_idx: Idx,
@@ -61,7 +61,7 @@ impl<K: Ord, V> SGTree<K, V> {
 
     /// The [original scapegoat tree paper's](https://people.csail.mit.edu/rivest/pubs/GR93.pdf) alpha, `a`, can be chosen in the range `0.5 <= a < 1.0`.
     /// `a` tunes how "aggressively" the data structure self-balances.
-    /// It controls the trade-off between total rebuilding time and maximum height guarantees.
+    /// It controls the trade-off between total rebuild time and maximum height guarantees.
     ///
     /// * As `a` approaches `0.5`, the tree will rebalance more often. Ths means slower insertions, but faster lookups and deletions.
     ///     * An `a` equal to `0.5` means a tree that always maintains a perfect balance (e.g."complete" binary tree, at all times).
@@ -93,6 +93,26 @@ impl<K: Ord, V> SGTree<K, V> {
         }
     }
 
+    /// Get the current rebalance parameter, alpha, as a tuple of `(alpha_numerator, alpha_denominator)`.
+    /// See [the corresponding setter method][SGTree::set_rebal_param] for more details.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scapegoat::SGTree;
+    ///
+    /// let mut sgt: SGTree<isize, isize> = SGTree::new();
+    ///
+    /// // Set 2/3, e.g. `a = 0.666...` (it's default value).
+    /// assert!(sgt.set_rebal_param(2.0, 3.0).is_ok());
+    ///
+    /// // Get the currently set value
+    /// assert_eq!(sgt.rebal_param(), (2.0, 3.0));
+    /// ```
+    pub fn rebal_param(&self) -> (f32, f32) {
+        (self.alpha_num, self.alpha_denom)
+    }
+
     /// `#![no_std]`: total capacity, e.g. maximum number of tree pairs.
     /// Attempting to insert pairs beyond capacity will panic, unless the `high_assurance` feature is enabled.
     ///
@@ -997,8 +1017,8 @@ impl<K: Ord, V> SGTree<K, V> {
             return None;
         }
 
-        let mut node_subtree_size = 1;
-        let mut parent_path_idx = path.len() - 1;
+        let mut node_subtree_size = 1; // Newly inserted
+        let mut parent_path_idx = path.len() - 1; // Parent of newly inserted
         let mut parent_subtree_size = self.get_subtree_size(path[parent_path_idx]);
 
         while (parent_path_idx > 0)
@@ -1007,7 +1027,11 @@ impl<K: Ord, V> SGTree<K, V> {
         {
             node_subtree_size = parent_subtree_size;
             parent_path_idx -= 1;
-            parent_subtree_size = self.get_subtree_size(path[parent_path_idx]);
+            parent_subtree_size = self.get_subtree_size_differential(
+                path[parent_path_idx],     // Parent index
+                path[parent_path_idx + 1], // Child index
+                node_subtree_size,         // Child subtree size
+            );
 
             debug_assert!(parent_subtree_size > node_subtree_size);
         }
@@ -1024,15 +1048,19 @@ impl<K: Ord, V> SGTree<K, V> {
         }
 
         let mut i = 0;
-        let mut node_subtree_size = 1;
-        let mut parent_path_idx = path.len() - 1;
+        let mut node_subtree_size = 1; // Newly inserted
+        let mut parent_path_idx = path.len() - 1; // Parent of newly inserted
         let mut parent_subtree_size = self.get_subtree_size(path[parent_path_idx]);
 
         while (parent_path_idx > 0) && (i <= self.alpha_balance_depth(node_subtree_size)) {
             node_subtree_size = parent_subtree_size;
             parent_path_idx -= 1;
             i += 1;
-            parent_subtree_size = self.get_subtree_size(path[parent_path_idx]);
+            parent_subtree_size = self.get_subtree_size_differential(
+                path[parent_path_idx],     // Parent index
+                path[parent_path_idx + 1], // Child index
+                node_subtree_size,         // Child subtree size
+            );
 
             debug_assert!(parent_subtree_size > node_subtree_size);
         }
@@ -1067,6 +1095,58 @@ impl<K: Ord, V> SGTree<K, V> {
         self.arena.hard_get(idx).subtree_size
     }
 
+    // Differential subtree size helper
+    #[cfg(not(feature = "fast_rebalance"))]
+    fn get_subtree_size_differential(
+        &self,
+        parent_idx: Idx,
+        child_idx: Idx,
+        child_subtree_size: Idx,
+    ) -> Idx {
+        let parent = self.arena.hard_get(parent_idx);
+
+        debug_assert!(
+            (parent.right_idx == Some(child_idx)) || (parent.left_idx == Some(child_idx))
+        );
+
+        let mut is_right_child = false;
+        if let Some(right_child_idx) = parent.right_idx {
+            if right_child_idx == child_idx {
+                is_right_child = true;
+            }
+        }
+
+        let other_child_subtree_size = if is_right_child {
+            match parent.left_idx {
+                Some(idx) => self.get_subtree_size(idx),
+                None => 0,
+            }
+        } else {
+            match parent.right_idx {
+                Some(idx) => self.get_subtree_size(idx),
+                None => 0,
+            }
+        };
+
+        let computed_subtree_size = child_subtree_size + other_child_subtree_size + 1;
+
+        debug_assert_eq!(computed_subtree_size, self.get_subtree_size(parent_idx));
+
+        computed_subtree_size
+    }
+
+    // Subtree size helper
+    // Size already cached if `fast_rebalance` is enabled, no need for differential logic
+    #[cfg(feature = "fast_rebalance")]
+    fn get_subtree_size_differential(
+        &self,
+        parent_idx: Idx,
+        _child_idx: Idx,
+        _child_subtree_size: Idx,
+    ) -> Idx {
+        self.get_subtree_size(parent_idx)
+    }
+
     // Iterative in-place rebuild for balanced subtree
     fn rebuild(&mut self, idx: Idx) {
         let sorted_sub = self.flatten_subtree_to_sorted_idxs(idx);
@@ -1350,9 +1430,9 @@ impl<'a, K: Ord, V> IntoIterator for &'a SGTree<K, V> {
 // Consuming iterator
 impl<K: Ord, V> IntoIterator for SGTree<K, V> {
     type Item = (K, V);
-    type IntoIter = ConsumingIter<K, V>;
+    type IntoIter = IntoIter<K, V>;
 
     fn into_iter(self) -> Self::IntoIter {
-        ConsumingIter::new(self)
+        IntoIter::new(self)
     }
 }
